'''
Helper funations to be used.
'''

import math, copy, csv
import numpy as np

print_it = True

def print_helper(properties, prop_text, units):
    '''
    Used to print out the properties
    '''
    dummy_i = 0
    print(' \n ')
    for prop_i in prop_text:
        print(str(prop_i) + ' ' + str(properties[dummy_i]) + ' ' + str(units[dummy_i]) )
        dummy_i += 1

def dist(p, q):
    return math.sqrt((p[0] - q[0]) ** 2 + (p[1] - q[1]) ** 2)

def get_num(x):
    try:
        return int(''.join(ele for ele in x if ele.isdigit() or ele == '.'))
    except ValueError:
        return x

def list_2_string(list):
    new_string = ''
    for item in list[0:-1]:
        new_string += str(item)
        new_string += ' , '
    new_string += str(list[-1])
    return new_string

def one_load_combination(line_name_obj, coord, defined_loads, load_condition,
                         defined_tanks, comb_name, acc, load_factors_all):
    '''
    Creating load combination.
    Inserted into self.line_to_struc index = 4
    "dnva", "line12", "static_ballast_10m"
    #load combination dictionary (comb,line,load) : [stat - DoubleVar(), dyn - DoubleVar], on/off - IntVar()]
    :return:
    '''

    if load_condition not in ['tanktest','manual', 'slamming']:
        return helper_dnva_dnvb(line_name_obj, coord, defined_loads, load_condition,
                                defined_tanks, comb_name, acc, load_factors_all)
    elif load_condition ==  'tanktest' and comb_name == 'tanktest':
        return helper_tank_test(line_name_obj, coord, defined_loads, load_condition,
                                defined_tanks, comb_name, acc, load_factors_all)
    elif load_condition == 'manual':
        return helper_manual(line_name_obj, comb_name,load_factors_all)
    elif load_condition == 'slamming':
        return helper_slamming(defined_loads)
    else:
        return [None, ' ']

def helper_dnva_dnvb(line_name_obj, coord, defined_loads, load_condition,
                         defined_tanks, comb_name, acc, load_factors_all):

    # calculate the defined loads
    calc_load, load_print, prt_conditions = [], ['',], []
    line_name = line_name_obj[0]
    structure_type = line_name_obj[1].get_structure_type()
    # if line_name_obj[0] == 'line12':
    #     print('Load calculation for '+line_name_obj[0] + ' ' + comb_name+ ' ' + load_condition)

    # if print_line != None and line_name == print_line:
    #     print('Load calculation for '+line_name_obj[0] + ' ' + comb_name+ ' ' + load_condition)

    if len(defined_loads) !=  0:
        for load in defined_loads :
            if comb_name+load_condition not in prt_conditions:
                load_print.append("Loads for condition: " + load_condition + ' - ' + comb_name +' ' + '\n')
                prt_conditions.append(comb_name+load_condition)
            if load != None:

                load_factors = load_factors_all[(comb_name, line_name, load.get_name())]
                # if print_it:
                #     if load_factors[0].get() != 0:
                #         load_print.append('LOAD NAME: '+' '+ comb_name+ ' '+ line_name+' '+ load.get_name()+'\n')
                # if load_factors[1].get() != 0:
                #     if load_factors[0].get() != 0:
                #         load_print.append('LOAD NAME: '+' '+ comb_name+ ' '+ line_name+' '+ load.get_name()+'\n')
                # USE GET() (static,dyn, on/off)

                if load_condition == load.get_load_condition():
                    static_pressure = (load_factors[2].get())*(load_factors[0].get())\
                                      *load.get_calculated_pressure(coord, acc[0],structure_type)
                    dynamic_pressure = (load_factors[2].get())*(load_factors[1].get())\
                                       *load.get_calculated_pressure(coord, acc[1],structure_type)
                    if print_it:
                        # load_print.append('load (NON-TANK) calculation for load condition:' + load_condition + ' - Load is: '+ \
                        #      load.get_name() + ' - Type is: \n')
                        if load_factors[0].get() != 0:
                            load_print.append(' static with acceleration: '+ str(acc[0])+ ' is: \n '+
                                              str(load_factors[2].get())+'*'+\
                                 str(load_factors[0].get())+'*'+\
                                 str(round(load.get_calculated_pressure(coord, acc[0],structure_type),1))+ ' = '+ \
                                 str(round(static_pressure,1))+'\n')
                        if load_factors[1].get() != 0:
                            load_print.append(' dynamic with acceleration: '+ str(acc[1])+' is: \n '+
                                              str(load_factors[2].get())+'*'+\
                                 str(load_factors[1].get())+'*'+\
                                 str(round(load.get_calculated_pressure(coord, acc[1],structure_type),1))+ ' = '+ \
                                 str(round(dynamic_pressure,1))+'\n')
                    # if line_name_obj[0] == 'line12':
                    #     print('Pressures',static_pressure,'+',dynamic_pressure,'=',static_pressure+dynamic_pressure)
                    calc_load.append(static_pressure+dynamic_pressure)

    # calculate the tank loads

    if len(defined_tanks) != 0:
        temp_tank = {}
        if comb_name + load_condition not in prt_conditions:
            load_print.append("Loads for condition: " + load_condition + ' - ' + comb_name + ' ' + '\n')
            prt_conditions.append(comb_name + load_condition)
        for tank_name_obj in defined_tanks:
            temp_tank[tank_name_obj[0]] = 0

            load_factors = load_factors_all[(comb_name, line_name, tank_name_obj[0])]
            overpress_lf = [1.3,0]# if load_factors[0].get()==1.2 else [1,1.3]

            if load_condition == tank_name_obj[1].get_condition():
                # USE GET() (static,dyn, on/off)
                static_pressure = load_factors[2].get()*(load_factors[0].get())\
                                  *tank_name_obj[1].get_calculated_pressure(coord,acc[0])\
                                  +tank_name_obj[1].get_overpressure()*overpress_lf[0]
                dynamic_pressure = load_factors[2].get()*load_factors[1].get()\
                                   *tank_name_obj[1].get_calculated_pressure(coord,acc[1])\
                                   +tank_name_obj[1].get_overpressure()*overpress_lf[1]

                temp_tank[tank_name_obj[0]] = static_pressure + dynamic_pressure# .append((static_pressure + dynamic_pressure))
                if print_it and tank_name_obj[0]+load_condition not in prt_conditions:
                    prt_conditions.append(tank_name_obj[0]+load_condition)
                    #load_print.append('load (TANK) calculation for load condition:'+ load_condition+ ' - Tank is: '+ tank_name_obj[0]+'\n')
                    #load_print.append('load factors : '+ str(load_factors[0].get())+str(load_factors[1].get())+str(load_factors[2].get())+'\n')
                    load_print.append('\n' + tank_name_obj[0] + ' - static: '+ str(load_factors[2].get())+ '*'+ str(load_factors[0].get()) + '*'+\
                          str(tank_name_obj[1].get_calculated_pressure(coord,acc[0]))+' + '+\
                          str(tank_name_obj[1].get_overpressure())+ '*'+str(overpress_lf[0])+ ' = '+str(static_pressure)+'\n')
                    load_print.append(tank_name_obj[0] + ' - dynamic: '+str(load_factors[2].get())+ '*'+ str(load_factors[1].get())+  '*'+\
                          str(tank_name_obj[1].get_calculated_pressure(coord, acc[1]))+' + '+\
                          str(tank_name_obj[1].get_overpressure())+ '*'+str(overpress_lf[1])+' = '+ str(dynamic_pressure)+'\n')
            # choosing the tank with the highest pressures

        if len(defined_loads) == 0:
            line_tank_pressure_calc = max([pressure for pressure in temp_tank.values()])

            #print('line_tank_pressure_calc', line_tank_pressure_calc)
            highest_dnv_tank_pressure  = tank_name_obj[1].get_tank_dnv_minimum_pressure(load_factors[0].get(),
                                                                                        load_factors[1].get())
            #print('highest_dnv_tank_pressure', highest_dnv_tank_pressure)
            line_dnv_tank_pressure = tank_name_obj[1].get_line_pressure_from_max_pressure(highest_dnv_tank_pressure,
                                                                                          coord)
            #print('line_dnv_tank_pressure', line_dnv_tank_pressure)
            # if line_name_obj[0] == 'line29':
            #     print('Tank load to append is max( ',highest_tank_pressure_calc,highest_dnv_tank_pressure,')')
            highest_tank_pressure = max(line_tank_pressure_calc,line_dnv_tank_pressure)
            calc_load.append(-highest_tank_pressure if highest_tank_pressure else 0)
            load_print.append('\nDNVGL-OS-C101 4.3.7 and 4.3.8 (Tank pressures) = '+ str(highest_tank_pressure)+'\n')
        else:
            pass
    if print_it:
        if len(calc_load) == 2:
            load_print.append('\nRESULT: ' + str(round(calc_load[0], 1)) +' + '+
                              str(round(calc_load[1])) + ' = ' + str(round(sum(calc_load),1)) +'\n')
        elif len(calc_load) == 1:
            load_print.append(
                '\nRESULT: ' + str(round(calc_load[0],1))+'\n')
        else:
            pass

        load_print.append('------------------------------------------------------------------\n')
    # if line_name_obj[0] == 'line12':
    #     print('end')
    return [int(abs(sum(calc_load))), load_print]

def helper_slamming(defined_loads):

    # calculate the defined loads
    calc_load, load_print = [], ['',]
    if len(defined_loads) != 0:
        for load in defined_loads:
            if load != None and load.get_load_condition() == 'slamming':
                load_print.append('Slamming pressure: \n'+ str(load.get_calculated_pressure(0, 0, 'slamming'))+ ' Pa \n')
                return [load.get_calculated_pressure(0, 0, 'slamming'), load_print]
    return [None, ' ']


def helper_tank_test(line_name_obj, coord, defined_loads, load_condition,
                                defined_tanks, comb_name, acc, load_factors_all):
    # calculate the defined loads
    calc_load, load_print = [], ['',]
    static_pressure, dynamic_pressure = 0, 0
    line_name = line_name_obj[0]
    structure_type = line_name_obj[1].get_structure_type()

    if len(defined_loads) != 0:
        for load in defined_loads:
            if load != None:
                load_factors = load_factors_all[(comb_name, line_name, load.get_name())]
                # USE GET() (static,dyn, on/off)
                if load_condition == load.get_load_condition():
                    static_pressure = (load_factors[2].get()) * (load_factors[0].get()) \
                                      * load.get_calculated_pressure(coord, acc[0], structure_type)
                    dynamic_pressure = (load_factors[2].get()) * (load_factors[1].get()) \
                                       * load.get_calculated_pressure(coord, acc[1], structure_type)
                    calc_load.append(static_pressure + dynamic_pressure)
                    if print_it:
                        load_print.append(
                            'Tank test for: ' + load_condition[0] + '\n' + str(load_factors[2].get())+' * '+
                            str(load_factors[0].get()) +' * '+
                            str(round(load.get_calculated_pressure(coord, acc[0], structure_type),1)) + ' + ' +
                            str(round(dynamic_pressure)) + ' = ' + str(round(dynamic_pressure + static_pressure))+'\n')


    # calculate the tank loads
    temp_tank={}
    if len(defined_tanks) != 0:

        for tank_name_obj in defined_tanks:
            temp_tank[tank_name_obj[0]] = []

        for tank_name_obj in defined_tanks:
            load_factors = load_factors_all[(comb_name, line_name, tank_name_obj[0])]
            # if print_it:
            #     load_print.append('Tank test LF: '+ str(load_factors[0].get())+' '+str(load_factors[1].get())+' '+
            #                       str(load_factors[2].get())+'\n')
                # USE GET() (static,dyn, on/off)
            overpress_lf = [1.3, 0] if load_factors[0].get() == 1.2 else [1, 0]
            static_pressure = (load_factors[2].get()) * (load_factors[0].get())\
                              * tank_name_obj[1].get_calculated_pressure(coord, acc[0])\
                              +tank_name_obj[1].get_overpressure()*overpress_lf[0]
            dynamic_pressure = (load_factors[2].get()) * (load_factors[1].get())\
                               * tank_name_obj[1].get_calculated_pressure(coord, acc[1])\
                               +tank_name_obj[1].get_overpressure()*overpress_lf[1]

            temp_tank[tank_name_obj[0]].append((static_pressure + dynamic_pressure))
            if print_it:
                load_print.append(
                    'Tank test for: ' + tank_name_obj[0] + '\n' + str(load_factors[2].get()) + ' * ' +
                    str(load_factors[0].get()) + ' * ' +
                    str(round(tank_name_obj[1].get_calculated_pressure(coord, acc[0]), 1)) + ' + ' +
                    str(tank_name_obj[1].get_overpressure()) +' * ' + str(overpress_lf[0]) +
                    ' = ' + str(round(dynamic_pressure + static_pressure)) + '\n')
        # choosing the tank with the highest pressures
        if len(defined_tanks) != 0:
            highest_tank_pressure = max([temp_tank[tank[0]] for tank in defined_tanks])
            calc_load.append(-highest_tank_pressure[0] if len(highest_tank_pressure) > 0 else 0)
        else:
            pass
    return [int(abs(sum(calc_load))), load_print]

def helper_manual(line_name, comb_name,load_factors_all):
    calc_load, load_print = [], ['',]
    if (comb_name, line_name[0], 'manual') not in load_factors_all.keys():
        return [0, 'Manual pressure: 0']
    load_factors = load_factors_all[(comb_name, line_name[0], 'manual')]
    man_press = load_factors[0].get() * load_factors[1].get() * load_factors[2].get()
    if print_it:
        load_print.append('Manual pressure:\n'+ str(load_factors[0].get())+' * '+ str(load_factors[1].get())+' * '+
                          str(load_factors[2].get()) + ' = '+ str(man_press) +'\n')
    return [man_press, load_print]

def helper_read_section_file(files, obj = None, to_json = False, to_csv = None):
    ''' Read a xml file. '''
    import json
    from xml.dom import minidom
    to_return_final, to_return, return_csv = list(),  dict(), list()
    if type(files) != list:
        files = [files,]
    for file in files:
        if file.endswith('xml'):
            xmldoc = minidom.parse(file)
            sectionlist = xmldoc.getElementsByTagName('section')
            sec_types = ('unsymmetrical_i_section', 'l_section', 'bar_section')

            for idx, sec_type in enumerate(sec_types):
                sec_type_get = xmldoc.getElementsByTagName(sec_type)
                if sec_types == []:
                    continue
                for item, itemdata in zip(sectionlist, sec_type_get):
                    if sec_type == sec_types[0]:
                        stf_web_h, stf_web_thk = 'h', 'tw'
                        stf_flange_width, stf_flange_thk  = 'bfbot', 'tfbot'
                        stiffener_type = 'T'
                        mult = 1/1000
                    elif sec_type == sec_types[1]:
                        stf_web_h, stf_web_thk = 'h', 'tw'
                        stf_flange_width, stf_flange_thk  = 'b', 'tf'
                        stiffener_type = 'L'
                        mult = 1/1000
                    elif sec_type == sec_types[2]:
                        stf_web_h, stf_web_thk = 'h', 'b'
                        stf_flange_width, stf_flange_thk  = None, None
                        stiffener_type = 'FB'
                        mult = 1 / 1000
                    section_name = item.getAttribute('name')
                    to_return[section_name] = {'stf_web_height': [float(itemdata.getAttribute(stf_web_h)) *mult, 'm'],
                                               'stf_web_thk': [float(itemdata.getAttribute(stf_web_thk)) *mult,'m'],
                                               'stf_flange_width': [0 if stf_flange_width is None else
                                               float(itemdata.getAttribute(stf_flange_width)) *mult,'m'],
                                               'stf_flange_thk': [0 if stf_flange_thk is None else
                                               float(itemdata.getAttribute(stf_flange_thk)) *mult, 'm'],
                                               'stf_type': [stiffener_type, '']}

                    return_csv.append([to_return[section_name][var][0] for var in ['stf_web_height', 'stf_web_thk',
                                                                                   'stf_flange_width', 'stf_flange_thk',
                                                                                   'stf_type']])
            if to_json:
                with open('sections.json', 'w') as file:
                    json.dump(to_return, file)
            if to_csv:
                with open('sections.csv', 'w', newline='') as file:
                    section_writer = csv.writer(file)
                    for line in return_csv:
                        section_writer.writerow(line)

        elif file.endswith('json'):
            with open(file, 'r') as json_file:
                to_return = json.load(json_file)

        elif file.endswith('csv'):
            with open(file, 'r') as csv_file:
                csv_reader = csv.reader(csv_file, delimiter=',')
                for idx, section in enumerate(csv_reader):
                    if section[4] in ['L-bulb', 'bulb', 'hp']:
                        to_return[str(idx)] = {'stf_web_height': [float(section[0]) - float(section[3]), 'm'],
                                               'stf_web_thk': [float(section[1]),'m'],
                                               'stf_flange_width': [float(section[2]),'m'],
                                               'stf_flange_thk': [float(section[3]), 'm'],
                                               'stf_type': [section[4], '']}
                    else:
                        to_return[str(idx)] = {'stf_web_height': [float(section[0]), 'm'],
                                               'stf_web_thk': [float(section[1]),'m'],
                                               'stf_flange_width': [float(section[2]),'m'],
                                               'stf_flange_thk': [float(section[3]), 'm'],
                                               'stf_type': [section[4], '']}

    if to_json:
        with open('sections.json', 'w') as file:
            json.dump(to_return, file)

    if to_csv is not None:
        with open(to_csv, 'w', newline = '') as file:
            section_writer = csv.writer(file)
            for line in return_csv:
                section_writer.writerow(line)
    if obj is not None:  # This will return a modified object.
        if type(obj) is not list:
            obj = [obj, ]
            append_list = [[],]
        else:
            append_list = [list() for dummy in obj]
    else:
        append_list = list()

    for key, value in to_return.items():
        if obj is not None:  # This will return a modified object.
            for idx, iter_obj in enumerate(obj):
                new_obj = copy.deepcopy(iter_obj)
                new_obj_prop = new_obj.get_structure_prop()
                for prop_name, prop_val in value.items():
                    new_obj_prop[prop_name] = prop_val
                new_obj.set_main_properties(new_obj_prop)
                append_list[idx].append(new_obj)
        else:
            to_return_final.append(value)
    if len(append_list) == 1:
        to_return_final = append_list[0]
    elif len(append_list) == 0:
        pass
    elif len(append_list) > 1:
        to_return_final = append_list

    return to_return_final

def open_example_file(root_path = None):
    import os

    if os.path.isfile('sections.csv'):
        os.startfile('sections.csv')
    else:
        os.startfile(root_path + '/' + 'sections.csv')

def add_new_section(section_list, new_section):
    ''' Checking if a section is already in the list. '''
    existing_section = False

    for section in section_list:

        if section.__str__() == new_section.__str__():
            existing_section = True

    if existing_section == False:
        # print('The new section', new_section)
        # print('The section list', section_list)
        section_list.append(new_section)

    return section_list

def plot_weights(time_stamp = None, cog = None, structure = None, weight = None):
    if __name__ == '__main__':
        cog = [[22.15329254, 12.24742235],
               [22.1937807, 12.1975691],
               [22.24684556, 12.15423614],
               [22.27489223, 12.09378247],
               [22.29086617, 12.03458725],
               [22.29559601, 11.97667798],
               [22.58758899, 11.739118],
               [22.34550004, 11.936077],
               [22.39332625, 11.96360235],
               [22.43016887, 11.99128875],
               [22.29320631, 12.02004097],
               [22.2458229, 11.99243978],
               [22.20984338, 11.96499817]]
        cog = np.array(cog)
        structure = {'plates': [18.0, 25.0, 12.0, 20.0, 14.0, 30.0, 15.0],
                     'beams': ['T_400_0x12_0__200_0x20_0', 'T_400_0x12_0__250_0x14_0', 'T_400_0x12_0__250_0x12_0',
                               'T_400_0x12_0__200_0x18_0', 'T_400_0x12_0__150_0x20_0', 'T_500_0x12_0__150_0x20_0',
                               'T_340_0x12_0__200_0x20_0', 'T_340_0x12_0__150_0x16_0', 'T_250_0x12_0__150_0x14_0',
                               'T_450_0x12_0__150_0x20_0', 'T_375_0x12_0__150_0x18_0', 'T_500_0x12_0__150_0x25_0',
                               'T_325_0x12_0__150_0x16_0', 'FB_250_0x18_0', 'FB_400_0x18_0',
                               'T_350_0x12_0__150_0x20_0', 'T_320_0x12_0__150_0x20_0', 'T_300_0x12_0__150_0x20_0']}

        time_stamp = [18920.477643045164, 18920.477684256162, 18920.477721255855, 18920.477761896746, 18920.477798285963,
                      18920.477841150896, 18920.4778763735, 18920.477939357952, 18920.47800752034, 18920.47808087777,
                      18920.478203353003, 18920.478237156338, 18920.47826686926]
        weight = [0.97156037, 0.97553128, 0.979408, 0.97625964, 0.97319636, 0.97021818,
                  1., 0.97518182, 0.97234545, 0.96950909, 0.97546545, 0.97830182,
                  0.98113818]

    import matplotlib.dates as mdate
    from matplotlib import pyplot as plt
    from matplotlib.gridspec import GridSpec

    pl_and_bm = [['', ''] for dummy in range(max(len(list(structure.values())[0]), len(list(structure.values())[1])))]

    for key, value in structure.items():
        for idx, val in enumerate(value):
            if key == 'plates':
                pl_and_bm[idx][0] = str(val) + ' mm'
            else:
                pl_and_bm[idx][1] = val

    fig = plt.figure(figsize=(14, 8))
    gs = GridSpec(2, 3, figure=fig)

    time_stamp = [mdate.epoch2num(val) for val in time_stamp]

    ax3 = plt.subplot(gs[1, 0:2])
    plt.plot(time_stamp, weight, 'tab:green')

    ax1 = plt.subplot(gs[0, 0], sharex=ax3)
    plt.plot(time_stamp, cog[:, 0])

    ax2 = plt.subplot(gs[0, 1], sharex=ax3)
    plt.plot(time_stamp, cog[:, 1], 'tab:orange')

    ax4 = plt.subplot(gs[0:2, 2])
    ax4.set_axis_off()
    table1 = plt.table(cellText=pl_and_bm, colLabels = ['Plates in model', 'Beams in model'],loc='center')
    table1.auto_set_column_width((0,1))
    table1.scale(1,1.5)
    # ax5 = plt.subplot(gs[0:2, 3])
    # ax5.set_axis_off()
    # table2 = plt.table(cellText=structure['beams'], colLabels = ['Beams in model'],loc='center')
    # table2.scale(1,1.5)
    # table2.auto_set_column_width(0)
    # Choose your xtick format string
    date_fmt = '%d-%m-%y %H:%M:%S'

    # Use a DateFormatter to set the data to the correct format.
    date_formatter = mdate.DateFormatter(date_fmt)
    ax1.xaxis.set_major_formatter(date_formatter)
    ax2.xaxis.set_major_formatter(date_formatter)
    ax3.xaxis.set_major_formatter(date_formatter)
    ax1.set_title('COG X')
    ax2.set_title('COG Y')
    ax3.set_title('Total weight / max(total weight)')

    fig.suptitle('Developement of weight and COG')

    # Sets the tick labels diagonal so they fit easier.
    fig.autofmt_xdate()

    plt.tight_layout()
    plt.show()

def helper_cylinder_stress_to_force_to_stress(stresses = None, forces = None, geometry = None, shell_t = 0,
                                              shell_radius = 0, shell_spacing = 0,
                                              hw = 0, tw = 0, b = 0, tf = 0, CylinderAndCurvedPlate = None,
                                              conical = False, psd = 0, cone_r1 = 0, cone_r2 = 0, cone_alpha = 0,
                                              shell_lenght_l = 0):
    
    
    A = 0 if geometry in [1, 2] else hw * tw + b * tf
    eq_thk = shell_t if geometry in [1, 2] else shell_t + A/shell_spacing

    Itot = CylinderAndCurvedPlate.get_Itot(hw=0 if geometry in [1, 2] else hw,
                                           tw=0 if geometry in [1, 2] else tw,
                                           b=0 if geometry in [1, 2] else b,
                                           tf=0 if geometry in [1, 2] else tf,
                                           r=shell_radius,
                                           s=shell_spacing,
                                           t=shell_t)

    if forces is not None and stresses is None:
        if not conical:
            Nsd, Msd, Tsd, Qsd = forces
            sasd = (Nsd / 2) / (math.pi * shell_radius * eq_thk) * 1000
            smsd = (Msd/ Itot) * \
                   (shell_radius + shell_t / 2) * 1000000
            tTsd = (Tsd* 10 ** 6) / (2 * math.pi * shell_t * math.pow(shell_radius, 2))
            tQsd = Qsd / (math.pi * shell_radius * shell_t) * 1000
            shsd = 0
            return sasd, smsd, tTsd, tQsd, shsd
        else:
            Nsd, M1sd, M2sd, Tsd, Q1sd, Q2sd = forces
            re = (cone_r1+cone_r2) / (2*math.cos(math.radians(cone_alpha)))
            le = shell_lenght_l / math.cos(math.radians(cone_alpha))
            te = shell_t *math.cos(math.radians(cone_alpha))
            sasd = psd*re/2*te + Nsd/(2*math.pi*re*te) * 1000
            smsd = ((M1sd*math.sin(math.radians(cone_alpha)) / (math.pi*math.pow(re,2)*te)) + \
                   (M2sd*math.cos(math.radians(cone_alpha)) / (math.pi*math.pow(re,2)*te))) * 1000000
            shsd = psd*re/te
            tTsd = Tsd/(2*math.pi*math.pow(re,2)*te)
            tQsd = -(Q1sd*math.cos(math.radians(cone_alpha)) / (math.pi*re*te)) + \
                   (Q2sd*math.sin(math.radians(cone_alpha)) / (math.pi*re*te))
            return sasd, smsd, tTsd, tQsd, shsd

    else:
        if not conical:
            sasd, smsd, tTsd, tQsd, shsd = stresses
            Nsd = (sasd * 2 * math.pi * shell_radius * eq_thk) / 1000
            Msd = (smsd / (shell_radius * shell_t / 2)) * Itot / 1000000
            Tsd = tTsd * 2 * math.pi * shell_t * math.pow(shell_radius, 2) / 1000000
            Qsd = tQsd * math.pi * shell_radius * shell_t / 1000
        else:
            re = (cone_r1+cone_r2) / (2*math.cos(math.radians(cone_alpha)))
            le = shell_lenght_l / math.cos(math.radians(cone_alpha))
            te = shell_t *math.cos(math.radians(cone_alpha))
            Itot = CylinderAndCurvedPlate.get_Itot(hw=0,
                                                   tw=0 ,
                                                   b=0 ,
                                                   tf=0,
                                                   r=re,
                                                   s=shell_spacing,
                                                   t=te)
            sasd, smsd, tTsd, tQsd, shsd = stresses
            Nsd = (sasd * 2 * math.pi * re * te) / 1000
            Msd = (smsd / (re * te / 2)) * Itot / 1000000
            Tsd = tTsd * 2 * math.pi * te * math.pow(re, 2) / 1000000
            Qsd = tQsd * math.pi * re * te/ 1000

        return Nsd, Msd, Tsd, Qsd, shsd


if __name__ == '__main__':
    from tkinter import *



    class AllTkinterWidgets:
        def __init__(self, master):
            frame = Frame(master, width=500, height=400, bd=1)
            frame.pack()

            iframe5 = Frame(frame, bd=2, relief=RAISED)
            iframe5.pack(expand=1, fill=X, pady=10, padx=5)
            c = Canvas(iframe5, bg='white', width=340, height=200)
            c.pack()

            height = 150
            radius = 150
            offset_oval = 30
            start_x_cyl = 150
            start_y_cyl = 20
            coord1 = start_x_cyl, start_y_cyl, start_x_cyl + radius, offset_oval
            coord2 = start_x_cyl, start_y_cyl + height, start_x_cyl + radius, offset_oval+ height

            arc_1 = c.create_oval(coord1, width = 5, fill = 'grey90')
            arc_2 = c.create_arc(coord2, extent = 180, start = 180,style=ARC, width = 3)

            line1 = c.create_line(coord1[0], coord1[1]+offset_oval/4,
                                  coord1[0], coord1[1]+height+offset_oval/4,
                                  width = 3)
            line2 = c.create_line(coord1[0]+radius, coord1[1]+offset_oval/4,
                                  coord1[0]+radius, coord1[1]+height+offset_oval/4,
                                  width = 3)
            num_stf = 10
            for line_num in range(1,num_stf,1):
                angle = 180 - 180/(num_stf) *line_num
                arc_x, arc_y = 1*math.cos(math.radians(angle)), 0.5*math.sin(math.radians(angle))
                arc_x = (arc_x + 1)/2

                line1 = c.create_line(coord1[0] + radius*arc_x,
                                      coord1[1] +2*arc_y*offset_oval/3,
                                      coord1[0] + radius*arc_x,
                                      coord1[1] + height +2*arc_y*offset_oval/3,fill = 'blue')
            num_ring_stiff = 5
            for ring_stf in range(1,num_ring_stiff+1,1):
                coord3 = coord1[0], coord1[1]+(height/(num_ring_stiff+1))*ring_stf,  \
                         start_x_cyl +radius, coord1[3]+ (height/(num_ring_stiff+1))*ring_stf,
                arc_2 = c.create_arc(coord3, extent=180, start=180, style=ARC, width=2,fill = 'orange', outline = 'orange')

            num_ring_girder = 1
            for ring_girder in range(1, num_ring_girder+1,1):
                coord3 = coord1[0], coord1[1]+(height/(num_ring_girder+1))*ring_girder,  \
                         start_x_cyl+ radius, coord1[3]+ (height/(num_ring_girder+1))*ring_girder,
                arc_2 = c.create_arc(coord3, extent=180, start=180, style=ARC, width=4, fill = 'grey', outline = 'grey')

            iframe5.pack(expand=1, fill=X, pady=10, padx=5)


    root = Tk()
    # root.option_add('*font', ('verdana', 10, 'bold'))
    all = AllTkinterWidgets(root)
    root.title('Tkinter Widgets')
    root.mainloop()



import numpy as np

class MyProblem:
     def __init__(self, dim,upper_bounds,lower_bounds):
         super(MyProblem, self).__init__()
         self.dim = dim
         self.upper = upper_bounds
         self.lower = lower_bounds

     def fitness(self, x):
         return [sum(x*x)-sum(np.sqrt(x))]

     def get_bounds(self):
         return self.upper, self.lower

     def get_name(self):
         return "Sphere Function"

     def get_extra_info(self):
         return "\tDimensions: " + str(self.dim)


# prob = pg.problem(any_tester(10))
# algo = pg.algorithm(pg.pso(gen = 100, omega = 0.7298, eta1 = 2.05, eta2 = 2.05, max_vel = 0.5, variant = 5,
#                            neighb_type = 2, neighb_param = 4, memory = False))
# algo.set_verbosity(10)
# pop = pg.population(prob, 100)
# pop = algo.evolve(pop)
# print(pop.problem)
# uda = algo.extract(pg.pso)
# print(uda.get_log())



'''
This file open up excel PULS file
1. read and write to the file.
2. run macros.
3. get the results.
'''

import numpy as np
from xlwings import App, Book

class PulsExcel():
    ''' This class open a PulsExcel work.
        Input and output structure data and results.
        Running macros.
    '''
    def __init__(self, path_and_file_to_book: str = None, visible = True):
        '''
        :param path_and_file_to_book:  Path and file name or just filename in base directory.
        :param visible: If excel shall open in windows or run in the background.
        '''
        super(PulsExcel, self).__init__()
        self.app = App(visible=visible)
        self.book = Book(path_and_file_to_book)

        self.names_sp = {'Identification': 1, 'Length of panel': 3, 'Stiffener spacing': 4, 'Plate thickness': 5,
                      'Number of primary stiffeners': 6, 'Stiffener type (L,T,F)': 7, 'Stiffener boundary': 8,
                      'Stiff. Height': 9, 'Web thick.': 10,'Flange width': 11, 'Flange thick.': 12, 'Tilt angle': 13,
                      'Number of sec. stiffeners': 14, 'Modulus of elasticity': 21, "Poisson's ratio": 22,
                      'Yield stress plate': 23, 'Yield stress stiffener': 24, 'Axial stress': 25, 'Trans. stress 1': 26,
                      'Trans. stress 2': 27, 'Shear stress': 28, 'Pressure (fixed)': 29, 'In-plane support': 30}

        self.names_up = {'Identification': 1, 'Length of plate': 3, 'Width of c': 4, 'Plate thickness': 5,
                         'Modulus of elasticity': 6, "Poisson's ratio":7, 'Yield stress plate': 8,
                         'Axial stress 1': 9, 'Axial stress 2': 10,'Trans. stress 1': 11,
                         'Trans. stress 2': 12, 'Shear stress': 13, 'Pressure (fixed)': 14, 'In-plane support': 15,
                         'Rot left': 16, 'Rot right': 17, 'Rot upper': 18, 'Rot lower': 19}

    def close_book(self, save = False):
        ''' Closing ass and book. '''
        if save:
            self.book.save()
        self.book.close()
        self.app.kill()

    def read_data(self, row_number: int = 1, column_number: int = 1):
        ''' Read one cell in the sheet
            Row and columns starts at 1 (not 0 as in python general)
        '''
        return self.book.sheets[1].range((row_number, column_number)).value

    def set_cell_value(self, row_number: int = 1, column_number: int = 1, cell_value = None, sheet_num = 1):
        '''
        Set values of a cell.
        :param row_number:
        :param column_number:
        :param cell_value:
        :return:
        '''
        #print('Row', row_number, 'Col', column_number, 'Cell', cell_value)
        self.book.sheets[sheet_num].range((row_number, column_number)).value = cell_value

    def set_one_row(self, row_number: int = 20, data_dict: dict = None):
        '''
        Set one row of values
        :param row_number: The row to set.
        :param data_dict: Data for one panel.
        :return:
        '''
        for name, col_num in self.names_sp.items():
            self.set_cell_value(row_number, col_num, data_dict[name])

    def set_multiple_rows(self, start_row:int = 20, data_dict: dict = None):
        '''

        :param start_row: First row to input.
        :param list_of_dicts: The data to be set.
        :return:
        '''

        row_number = start_row
        for id, data in data_dict.items():
            self.set_one_row(row_number, data)
            row_number += 1

    def set_multiple_rows_batch(self, data_dict: dict = None):
        '''

        :param start_row: First row to input.
        :param list_of_dicts: The data to be set.
        :return:
        '''

        sp_dict, up_dict = dict(), dict()
        for key, val in data_dict.items():
            if val['sp or up'] == 'SP':
                sp_dict[key] = val
            else:
                up_dict[key] = val
        if len(sp_dict)>0:
            for name, col_num in self.names_sp.items():
                start_row = 20
                self.book.sheets[1].range((start_row, col_num)).options(expand='table', transpose=True).value = \
                    [val[name] for val in sp_dict.values()]
        if len(up_dict) > 0:
            for name, col_num in self.names_up.items():
                start_row = 21
                self.book.sheets[4].range((start_row, col_num)).options(expand='table', transpose=True).value = \
                    [val[name] for val in up_dict.values()]

        return len(sp_dict)>0, len(up_dict)>0

    def calculate_panels(self, sp = True, up = False):
        ''' Calculate the panels in the sheet. '''
        if sp:
            run_macro_sp = self.app.macro('Sheet1.cmdCalculatePanels_Click')
            run_macro_sp()
        if up:
            run_macro_up = self.app.macro('Sheet3.CalculatePanelsU3')
            run_macro_up()

    def get_results_one_cell(self, row_number: int = 1, column_number: int = 1):
        ''' Return the results in one cell of a calculated panel. '''
        return self.book.sheets[2].range((row_number, column_number)).value

    def get_results_one_row(self, row_number: int = 15):
        '''
        Return one row.
        :param row_number:
        :return: dict : [value, unit] e.g. [3000, 'mm']
        '''
        return_dict = dict()
        return_dict['Identification'] = self.get_results_one_cell(row_number, column_number=1)
        # print('start')
        # for idx in range(3,74):
        #     if self.get_results_one_cell(11, column_number=idx) != None:
        #         return_dict[self.get_results_one_cell(11, column_number=idx)] = {}
        # print('stop')
        current_top_row = ''
        for idx in range(3,74):
            if self.get_results_one_cell(11, column_number=idx) != None:
                current_top_row = self.get_results_one_cell(11, column_number=idx)
                return_dict[current_top_row] = {}

            return_dict[current_top_row][self.get_results_one_cell(12, column_number=idx)] = \
                [self.get_results_one_cell(row_number, column_number=idx),
                 self.get_results_one_cell(14, column_number=idx)]
        return return_dict

    def get_all_results_batch(self, sp = True, up = False):

        spup = {(True, True) : ['SP', 'UP'], (True, False) : ['SP'], (False, True) : ['UP']}

        return_dict = {}
        for run in spup[(sp, up)]:
            all_ids  = self.book.sheets[2 if run == 'SP' else 5].range('A15').expand().value
            if type(all_ids) != list:
                all_ids = [all_ids]
            all_data = np.array(self.book.sheets[2 if run == 'SP' else 5].range('C12').expand().value)
            all_top_names = np.array(self.book.sheets[2 if run == 'SP' else 5].range('C11:BU11' if run == 'SP' else 'C11:AS11').value)
            all_names = all_data[0]
            all_data = all_data[3:]
            all_units = np.array(self.book.sheets[2 if run == 'SP' else 5].range('C14:BU14'if run == 'SP' else 'C14:AS14').value)
            current_top_row = ''

            for data_idx, id in enumerate(all_ids):
                return_dict[id] = {}
                return_dict[id]['Identification'] = id
                same, same_idx = False, 2
                for top_name, name, data, unit in zip(all_top_names, all_names, all_data[data_idx], all_units):
                    if top_name != None:
                        current_top_row = top_name
                        return_dict[id][current_top_row] = {}
                    if name in return_dict[id][current_top_row].keys():
                        same = True
                        name = name + ' ' +str(same_idx)
                        same_idx += 1
                    elif same == True and name not in return_dict[id][current_top_row].keys():
                        same = False
                        same_idx = 2
                    return_dict[id][current_top_row][name] = [data, unit]
                if run == 'UP':
                    return_dict[id]['Ultimate capacity']['Actual usage Factor'] = \
                        return_dict[id]['Ultimate capacity']['Usage factor']
                    return_dict[id]['Ultimate capacity']['Allowable usage factor'] = \
                        return_dict[id]['Ultimate capacity']['Alowable usage']
                    return_dict[id]['Buckling strength']['Actual usage Factor'] = \
                        return_dict[id]['Buckling strength']['Usage factor']
                    return_dict[id]['Buckling strength']['Allowable usage factor'] = \
                        return_dict[id]['Buckling strength']['Alowable usage']

                    return_dict[id]['Ultimate capacity'].pop('Usage factor')
                    return_dict[id]['Ultimate capacity'].pop('Alowable usage')
                    return_dict[id]['Buckling strength'].pop('Usage factor')
                    return_dict[id]['Buckling strength'].pop('Alowable usage')
        return return_dict

    def get_all_results(self):
        '''
        Return all results in run.
        :return:
        '''

        return_dict, found_last, row_number = dict(), False, 15
        while found_last is False:
            this_row = self.get_results_one_row(row_number)
            if this_row['Identification'] is not None:
                return_dict[row_number] = this_row
                row_number += 1
            else:
                found_last = True

        return return_dict

if __name__ == '__main__':
    ex1 = {'line25': {'Identification': 'line25', 'Length of plate': 3, 'Width of c': 800.0, 'Plate thickness': 20.0,
                      'Modulus of elasticity': 210000.0, "Poisson's ratio": 0.3, 'Yield stress plate': 355.0,
                      'Axial stress 1': 60.0, 'Axial stress 2': 60.0, 'Trans. stress 1': 0, 'Trans. stress 2': 0,
                      'Shear stress': 10.0, 'Pressure (fixed)': 0.0, 'In-plane support': 'GL', 'Rot left': 'SS',
                      'Rot right': 'SS', 'Rot upper': 'SS', 'Rot lower': 'SS', 'sp or up': 'UP'}}
    my_puls = PulsExcel(r'C:\Github\ANYstructure\ANYstructure\PULS\PulsExcel_new - Copy.xlsm',
                        visible=True)
    my_puls.set_multiple_rows_batch(ex1)
    # my_puls.set_multiple_rows(20, [ex.ex1, ex.ex2, ex.ex3, ex.ex4, ex.ex5, ex.ex6])
    # my_puls.calculate_panels()
    #my_puls.set_one_row(data_dict=ex.ex1['line1'])
    # [print(key, value) for key, value in my_puls.get_all_results().items()]
    # my_puls.close_book()





#Making reports for the project

import time
from reportlab.lib.enums import TA_LEFT
from reportlab.platypus import Spacer
from reportlab.lib.styles import  ParagraphStyle
from PIL import Image
from reportlab.lib.pagesizes import letter, A4, landscape
from reportlab.lib.styles import getSampleStyleSheet
from reportlab.lib.units import mm, inch
from reportlab.pdfgen import canvas
from reportlab.platypus import Image, Paragraph, Table
from time import strftime, gmtime
import os
from matplotlib import pyplot as plt
import matplotlib
cmap_sections = plt.get_cmap('jet')
from reportlab.platypus import SimpleDocTemplate, TableStyle
from reportlab.lib import colors
from matplotlib import colors as matplotlib_colors
import tkinter as tk

try:
    import any_files.example_data as test
    import any_files.helper as hlp
except ModuleNotFoundError:
    import ANYstructure.any_files.example_data as test
    import ANYstructure.any_files.helper as hlp



def create_report(input_data):
    '''
    This class uses the module REPORTLAB to generate a report.
    :param line_obj_dict: 
    :type a dictionary
    :return: 
    '''

    Story = []

    file_name = "Report_current_results.pdf"
    doc = SimpleDocTemplate(file_name, pagesize=letter,
                            rightMargin=72, leftMargin=72,
                            topMargin=72, bottomMargin=18)

    # logo = "canvas_screenshot.gif"
    formatted_time = time.ctime()
    # im = Image(logo, 5 * inch, 4 * inch)
    # Story.append(im)
    styles = getSampleStyleSheet()
    styles.add(ParagraphStyle(name='Justify',alignment=TA_LEFT))
    ptext = '<font size=12>%s</font>' % formatted_time
    Story.append(Paragraph(ptext, styles["Normal"]))
    Story.append(Spacer(1, 12))

    # Create return address
    ptext = '<font size=12>%s</font>' % 'The results for the structure is shown here'
    Story.append(Paragraph(ptext, styles["Justify"]))
    Story.append(Spacer(1, 3))
    ptext = '----------------------------------------------------------------------------------------------------------'
    Story.append(Paragraph(ptext, styles["Justify"]))
    Story.append(Spacer(5, 3))

    for line in input_data['lines'].keys():
        struc_obj = input_data['calc_structure'][line]
        fat_obj = input_data['calc_fatigue'][line]
        pressure = input_data['pressures'][line]

        ptext = '<font size=12>' + 'Results for: '+str(line) + '</font>'
        Story.append(Paragraph(ptext, styles["Justify"]))

        ptext = '<font size=10>'+'Plate thickness: '+ str(struc_obj.Plate.get_pl_thk()*1000)+ ' [mm], Stiffener spacing: '+\
                str(struc_obj.get_s()*1000)+' [mm]'+'</font>'
        Story.append(Paragraph(ptext, styles["Justify"]))

        ptext = '<font size=10>'+'Stiffener: '+ str(struc_obj.Stiffener.get_web_h()*1000)+ 'x' + str(struc_obj.Stiffener.get_web_thk()*1000) \
                + ' + ' + str(struc_obj.Stiffener.get_fl_w()*1000)+ 'x' + str(struc_obj.Stiffener.get_fl_thk()*1000)  +'</font>'
        Story.append(Paragraph(ptext, styles["Justify"]))

        ptext = '<font size=10>'+struc_obj.get_report_stresses()+'</font>'
        Story.append(Paragraph(ptext, styles["Justify"]))

        ptext = '<font size=10>'+struc_obj.get_results_for_report()+'</font>'
        Story.append(Paragraph(ptext, styles["Justify"]))

    Story.append(Spacer(2, 3))
    ptext = '<font size=12>END OF RESULTS</font>'
    Story.append(Paragraph(ptext, styles["Justify"]))
    my_canvas = canvas.Canvas(file_name)
    my_canvas.line(0,0,200,200)
    doc.build(Story, canvasmaker=my_canvas)

class LetterMaker(object):
    """"""

    def __init__(self, pdf_file, org, seconds, data):

        self.c = canvas.Canvas(pdf_file, pagesize=A4)
        self.styles = getSampleStyleSheet()
        self.width, self.height = A4
        self.organization = org
        self.seconds = seconds
        self.data = data
        self.draw_lines()

    def createDocument(self):
        """"""
        voffset = 100
        user = os.getlogin()
        time_now = strftime("%a, %d %b %Y %H:%M:%S +0000", gmtime())

        # create return address
        address = """<font size="12"><strong> ANYstructure report generator<br/></strong></font>""" + '<br/>' + \
                  """<font size="12"> User: </font>""" + '<font size="12">' + user + '</font>' + '<br/>' + '<br/>' + \
                  """<font size="12"> Time : </font>""" + '<font size="12">' + time_now + '</font>' + '<br/>'+ \
                  '<br/>'+'<font size="12">' + self.data._project_information.get('1.0', tk.END) + '</font>'
        p = Paragraph(address, self.styles["Normal"])
        # add a logo and size it
        img_file_name = 'ANYstructure_logo.jpg'
        if os.path.isfile('images/' + img_file_name):
            file_path = 'images/' + img_file_name
        else:
            file_path = os.path.dirname(os.path.abspath(__file__)) + '/images/' + img_file_name
        logo = Image(file_path)
        logo.drawHeight = 1 * inch
        logo.drawWidth = 2.5 * inch
        data = [[p, logo]]
        table = Table(data, colWidths=4 * inch)
        table.setStyle([("VALIGN", (0, 0), (0, 0), "TOP")])
        table.wrapOn(self.c, self.width, self.height)
        table.drawOn(self.c, *self.coord(18, 50, mm))

        self.draw_lines()

        ptext = '<font size="12" color = "blue"><strong>' + "Compartments: " + '</strong></font>'
        self.createParagraph(ptext, 10, voffset + 85)
        delta = 0
        h_start = 130
        if self.data._tank_dict != {}:
            for name, obj in self.data._tank_dict.items():

                ptext = '<font size="7" color = "black">' + 'Name: '+ name + ', content: ' \
                        + obj.get_content() + '</font>'
                self.createParagraph(ptext, h_start, voffset + 100 + delta)

                delta += 3
                ptext = '<font size="7" color = "black">' + 'Min. elevation: ' + str(obj.get_lowest_elevation()) + \
                        ', Max. elevation: ' + str(obj.get_highest_elevation()) + '</font>'
                self.createParagraph(ptext, h_start, voffset + 100 + delta)

                delta += 3
                ptext = '<font size="7" color = "black">' + 'Applied overpressure: ' + str(obj.get_overpressure()) + \
                        '</font>'
                self.createParagraph(ptext, h_start, voffset + 100 + delta)

                delta += 3
                ptext = '<font size="7" color = "black">'+'(a_stat, a_dyn_loa, a_dyn_bal): ' + \
                        str(obj.get_accelerations()) + '</font>'
                self.createParagraph(ptext, h_start, voffset + 100 + delta)
                delta += 4
            try:
                self.c.drawImage('current_comps.png', 10,50, width=350, height=250)
            except OSError:
                self.c.drawImage('current_comps_NONE.png', 10, 50, width=350, height=250)

        # insert body of letter

        self.c.showPage()
        ptext = '<font size="12" color = "blue"><strong>' + "Results for defined structure: " + '</strong></font>'
        self.createParagraph(ptext, 10, 0)

        delta = 140 if self.data._new_buckling_method.get() == 'DNV-RP-C201 - prescriptive' else 180
        vpos = 950

        for line in sorted(self.data._line_dict.keys()):
            vpos -= delta

            if line in self.data._line_to_struc.keys():
                if self.data._line_to_struc[line][5] is None:
                    struc_obj = self.data._line_to_struc[line][0]
                    fo = self.data._line_to_struc[line][2]
                    pressure = self.data.get_highest_pressure(line)['normal']/1000
                    textobject = self.c.beginText()
                    textobject.setTextOrigin(30,vpos)
                    textobject.setFont("Helvetica-Oblique", 10)
                    textobject.textLine('*********** '+line+' ***********')
                    textobject.textLine('Plate thickness: '+ str(struc_obj.Plate.get_pl_thk()*1000)+ ' [mm]          '
                                                                                               'Stiffener spacing: '+
                                        str(struc_obj.Plate.get_s()*1000)+' [mm]'+ '          Span: '+
                                        str(round(struc_obj.Plate.get_span(),4))
                                        + ' [m]')
                    if struc_obj.Stiffener is not None:
                        textobject.textLine('Stiffener: '+ str(struc_obj.Stiffener.get_web_h()*1000)+ 'x' +
                                            str(struc_obj.Stiffener.get_web_thk()*1000)
                                            + ' + ' + str(struc_obj.Stiffener.get_fl_w()*1000)+ 'x' +
                                            str(struc_obj.Stiffener.get_fl_thk()*1000))

                        textobject.textLine('Fixation paramters: kps: = '+str(struc_obj.Plate.get_kps())+ '  kpp = '
                                            + str(struc_obj.Plate.get_kpp())+
                                            ', Bending moment factors km1/km2/km3 (support/field/support)' + ' = '+
                                            str(int(struc_obj.Plate.get_km1()))+'/'+
                                            str(int(struc_obj.Plate.get_km2()))+'/'+
                                            str(int(struc_obj.Plate.get_km3())))
                    textobject.textLine('Defined stresses [MPa]:  sigma_x1 = '+str(struc_obj.Plate.get_sigma_x1())+
                                        '  sigma_x2 = ' + str(struc_obj.Plate.get_sigma_x2()) +
                                        '  sigma_y1 = '+ str(struc_obj.Plate.get_sigma_y1()) +
                                        '  sigma_y2 = '+ str(struc_obj.Plate.get_sigma_y2()) +
                                        '  tau_xy = ' + str(struc_obj.Plate.get_tau_xy()))
                    textobject.textLine('ULS max pressure for line: '+ str(round(pressure,2)*1000)
                                        + ' [kPa]'+'      Pressure applied at: '+struc_obj.overpressure_side)

                    if fo is not None:
                        textobject.textLine('Fatigue pressure [Pa]: '+' p_int:'+' loaded/ballast/part = '
                                            + str(round(self.data.get_color_and_calc_state()['pressure_fls'][line]['p_int']['loaded'],0))
                                            +'/'+str(round(self.data.get_color_and_calc_state()['pressure_fls'][line]['p_int']['ballast'],0))
                                            +'/'+str(round(self.data.get_color_and_calc_state()['pressure_fls'][line]['p_int']['part'],0))
                                            + ' p_ext:'+' loaded/ballast/part = '+
                                            str(round(self.data.get_color_and_calc_state()['pressure_fls'][line]['p_ext']['loaded'],0))
                                            +'/'+str(round(self.data.get_color_and_calc_state()['pressure_fls'][line]['p_ext']['ballast'],0))
                                            +'/'+str(round(self.data.get_color_and_calc_state()['pressure_fls'][line]['p_ext']['part'],0)))
                    else:
                        textobject.textLine(' Fatigue pressure: No pressures defined')

                    textobject.setFillColor('red') if self.data.get_color_and_calc_state()['colors'][line]['section'] == 'red' \
                        else textobject.setFillColor('black')

                    textobject.textLine('Section modulus: '+str(int(min(self.data.get_color_and_calc_state()['section_modulus'][line]['sec_mod'])
                                                                    *1000**3))+ ' [mm3]'+'  Min. section modulus: '+
                                        str(int(self.data.get_color_and_calc_state()['section_modulus'][line]['min_sec_mod']*1000**3))+' [mm3]'+
                                        ' -> ' + 'OK' if int(min(self.data.get_color_and_calc_state()['section_modulus'][line]['sec_mod'])*1000**3) >=
                                                         int(self.data.get_color_and_calc_state()['section_modulus'][line]['min_sec_mod']*1000**3)
                                        else 'Section modulus: '+str(int(min(self.data.get_color_and_calc_state()['section_modulus'][line]['sec_mod'])
                                                                         *1000**3))+ ' [mm3]'+ '  Min. section modulus: '+
                                        str(int(self.data.get_color_and_calc_state()['section_modulus'][line]['min_sec_mod']*1000**3))+' [mm3]'+
                                        ' -> ' + 'NOT OK')
                    textobject.setFillColor('black')
                    textobject.setFillColor('red') if self.data.get_color_and_calc_state()['colors'][line]['thickness'] == 'red' \
                        else textobject.setFillColor('black')
                    textobject.textLine('Min plate thickness:  '+
                                        str(round(self.data.get_color_and_calc_state()['thickness'][line]['min_thk'],2)) + ' [mm] '
                                        ' -> ' +
                                        'OK' if struc_obj.Plate.get_pl_thk()*1000 >=
                                                self.data.get_color_and_calc_state()['thickness'][line]['min_thk'] else
                                        'Min plate thickness:  '+ str(round(
                                            self.data.get_color_and_calc_state()['thickness'][line]['min_thk'],2)) + ' [mm] '
                                        ' -> '+'NOT OK')
                    textobject.setFillColor('black')
                    textobject.setFillColor('red') if self.data.get_color_and_calc_state()['colors'][line]['shear'] == 'red' \
                        else textobject.setFillColor('black')
                    textobject.textLine('Shear area: '+str(int(self.data.get_color_and_calc_state()['shear_area'][line]['shear_area']*1000**2))+' [mm2] '+
                                        '   Min shear area: '+str(int(self.data.get_color_and_calc_state()['shear_area'][line]['min_shear_area']*1000**2))
                                        + ' [mm2] ' +
                                        ' -> ' + 'OK' if self.data.get_color_and_calc_state()['shear_area'][line]['shear_area'] >=
                                                         self.data.get_color_and_calc_state()['shear_area'][line]['min_shear_area']
                                        else 'Shear area: '+str(int(self.data.get_color_and_calc_state()['shear_area'][line]['shear_area']*1000**2))+
                                             ' [mm2] ' +
                                             '   Min shear area: '+str(int(self.data.get_color_and_calc_state()['shear_area'][line]['min_shear_area']*1000**2))
                                             + ' [mm2] ' + ' -> ' + 'NOT OK')
                    textobject.setFillColor('black')
                    if self.data._new_buckling_method.get() == 'DNV-RP-C201 - prescriptive':
                        textobject.setFillColor('red') if self.data.get_color_and_calc_state()['colors'][line]['buckling'] == 'red' \
                            else textobject.setFillColor('black')
                        buc_util = list()
                        for key, val in self.data.get_color_and_calc_state()['buckling'][line].items():
                            for uf in val.values():
                                if type(uf) == list:
                                    buc_util.append(uf[0])
                                    buc_util.append(uf[1])
                                else:
                                    buc_util.append(uf)

                        textobject.textLine('Highest buckling utilization DNV-RP-C203: '+
                                            str(round(max(buc_util),2))+
                                            ' -> '+'OK' if max(buc_util) < 1 else
                                            'Highest buckling utilization DNV-RP-C203: '+
                                            str(round(max(buc_util),2))+' -> '+'NOT OK')
                    elif self.data._new_buckling_method.get() == 'DNV PULS':
                        if self.data._PULS_results is not None:
                            puls_method = self.data._line_to_struc[line][0].Plate.get_puls_method()
                            textobject.textLine('PULS results using '+str(puls_method) + 'utilization with acceptance '+
                                                str(self.data._PULS_results.puls_acceptance))
                            if line in self.data._PULS_results.get_run_results().keys():
                                puls_buckling = self.data._PULS_results.get_run_results()[line]['Buckling strength']['Actual usage Factor'][0]
                                puls_ultimate = self.data._PULS_results.get_run_results()[line]['Ultimate capacity']['Actual usage Factor'][0]
                                if  puls_buckling is not None:
                                    if puls_method == 'buckling' and puls_buckling/self.data._PULS_results.puls_acceptance > 1:
                                        textobject.setFillColor('red')
                                textobject.textLine('PULS buckling utilization = ' + str(puls_buckling))
                                textobject.setFillColor('black')
                                if puls_ultimate is not None:
                                    if puls_method == 'ultimate'  and puls_ultimate/self.data._PULS_results.puls_acceptance > 1:
                                        textobject.setFillColor('red')
                                textobject.textLine('PULS ultimate utilization = ' + str(puls_ultimate))
                                textobject.setFillColor('black')

                    else:
                        puls_method = self.data._line_to_struc[line][0].Plate.get_puls_method()
                        textobject.textLine('ML-CL results using '+str(puls_method) + 'utilization with acceptance 0.87')
                        if line in self.data._PULS_results.get_run_results().keys():
                            ml_buckling = self.data.get_color_and_calc_state()['ML buckling class'][line]['buckling']
                            ml_ultimate = self.data.get_color_and_calc_state()['ML buckling class'][line]['ultimate']
                            color_ml_buc = self.data.get_color_and_calc_state()['ML buckling colors'][line]['buckling']
                            color_ml_ult = self.data.get_color_and_calc_state()['ML buckling colors'][line]['ultimate']
                            color_csr = self.data.get_color_and_calc_state()['ML buckling colors'][line]['CSR requirement']

                            if puls_method == 'buckling':
                                textobject.setFillColor('red' if color_ml_buc == 'red' else 'black')
                            textobject.textLine('Buckling ML-CL results: ' + self.data._ML_classes[ml_buckling])
                            textobject.setFillColor('black')
                            if puls_method == 'ultimate':
                                textobject.setFillColor('red' if color_ml_ult == 'red' else 'black')
                            textobject.textLine('Ultimate ML-CL result: ' + self.data._ML_classes[ml_ultimate])
                            textobject.setFillColor('red' if color_csr == 'red' else 'black')
                            textobject.textLine('CSR tank requirement (stiffener): ' + 'OK' if color_csr == 'green'
                                                else 'red')
                            textobject.setFillColor('black')

                    textobject.setFillColor('black')
                    textobject.setFillColor('red') if self.data.get_color_and_calc_state()['colors'][line]['fatigue'] == 'red' \
                        else textobject.setFillColor('black')
                    if self.data.get_color_and_calc_state()['fatigue'][line]['damage'] is not None:
                        textobject.textLine('Fatigue (plate/stiffeners) utilization: '+
                                            str(round(self.data.get_color_and_calc_state()['fatigue'][line]['damage'],2))+ ' * DFF('+
                                            str(self.data.get_color_and_calc_state()['fatigue'][line]['dff']) + ') = ' +
                                            str(round(self.data.get_color_and_calc_state()['fatigue'][line]['damage']*
                                                      self.data.get_color_and_calc_state()['fatigue'][line]['dff'],2)) + ' (SN-curve = '+
                                            self.data.get_color_and_calc_state()['fatigue'][line]['curve']+')')

                    else:
                        textobject.textLine('No fatigue results')

                    # textobject.textLine('Utilization percentage (highest calculated): '+
                    #                     str(int(max(self.data.get_color_and_calc_state()['utilization'][line].values())*100))+ '%')

                    textobject.setFillColor('black')
                    self.c.drawText(textobject)
                    vpos -= 10
                else:
                    cyl_obj = self.data._line_to_struc[line][5]
                    textobject = self.c.beginText()
                    textobject.setTextOrigin(30, vpos)
                    textobject.setFont("Helvetica-Oblique", 10)
                    textobject.textLine('*********** ' + line + ' ***********')
                    textobject.textLine('Cylinder radius: ' + str(round(cyl_obj.ShellObj.radius*1000,2)) +
                                        ' mm , thickness: ' +str(round(cyl_obj.ShellObj.thk*1000,2)) + ' mm')
                    textobject.textLine('Longitudinal stiffener: ' + cyl_obj.LongStfObj.get_beam_string())
                    textobject.textLine('Ring stiffener: ' + cyl_obj.LongStfObj.get_beam_string())
                    textobject.textLine('Heavy ring girder: ' + cyl_obj.LongStfObj.get_beam_string())
                    textobject.textLine(
                        'Dist. between rings/length, l: ' + str(round(cyl_obj.ShellObj.dist_between_rings, 1)))
                    textobject.textLine(
                        'Lenght of shell, L: ' + str(round(cyl_obj.ShellObj.length_of_shell, 1)) + ' '
                        +'Total cyl. lenght, Lc: ' + str(round(cyl_obj.ShellObj.tot_cyl_length, 1)))

                    results = cyl_obj.get_utilization_factors()
                    textobject.textLine('Design axial stress/force:     ' + str(cyl_obj.sasd / 1e6) + ' MPa')
                    textobject.textLine('Design bending stress/moment:  ' + str(cyl_obj.smsd / 1e6) + ' MPa')
                    textobject.textLine('Design tosional stress/moment: ' + str(cyl_obj.tTsd / 1e6) + ' MPa')
                    textobject.textLine('Design shear stress/force:     ' + str(cyl_obj.tQsd / 1e6) + ' MPa')
                    textobject.textLine('Design lateral pressure        ' + str(cyl_obj.psd / 1e6) + ' MPa' )
                    vpos -= 40
                    for key, value in results.items():
                        if key in ['Weight', 'Need to check column buckling']:
                            continue
                        if key not in ['Stiffener check', 'Stiffener check detailed']:
                            text_key = key
                            if key == 'Column stability check':
                                if results['Need to check column buckling'] == False:
                                    continue
                                uf_text = 'N/A' if value is None else 'OK' if value else 'Not ok'
                            else:
                                uf_text = 'N/A' if value is None else str(round(value, 2))
                            if value is None:
                                uf_col = 'grey'
                            else:
                                uf_col = 'red' if any([value > 1, value == False]) else 'green'
                            textobject.setFillColor(uf_col)
                            textobject.textLine(text_key + ' : UF = ' + uf_text)
                            textobject.setFillColor('black')
                        elif key == 'Stiffener check':
                            if value is not None:
                                textobject.textLine('Stiffener requirement checks:')
                                stf_type_all = ''
                                for stf_type, chk_bool in value.items():
                                    chk_text = 'OK' if chk_bool == True else 'Not OK' if chk_bool == False else 'N/A'
                                    stf_type_all += stf_type + ' : ' + chk_text + '     '
                                textobject.textLine(stf_type_all)
                        vpos -= 10

                    self.c.drawText(textobject)
                    vpos += 10
            else:
                textobject.setFont("Helvetica-Oblique", 10)
                textobject.textLine('*********** '+line+' ***********')
                textobject.textLine('(no structural properties defined)')
            if vpos <= 290:
                self.c.showPage()
                vpos = 950

    # ----------------------------------------------------------------------
        self.c.showPage()
        self.draw_lines(draw_type='section')
        self.c.showPage()
        self.draw_lines(draw_type='plate')
        self.c.showPage()
        self.draw_lines(draw_type='pressure')
        self.c.showPage()
        self.draw_lines(draw_type='utilization')
        self.c.showPage()
        self.draw_lines(draw_type='sigma x')
        self.c.showPage()
        self.draw_lines(draw_type='sigma y1')
        self.c.showPage()
        self.draw_lines(draw_type='sigma y2')
        self.c.showPage()
        self.draw_lines(draw_type='tau xy')
        self.c.showPage()
        self.draw_lines(draw_type='structure type')
        self.c.showPage()

        idx, new = 0, False
        for load_name in self.data._load_dict.keys():
            self.draw_lines(draw_type=None, load_idx_name = [idx % 3, load_name])
            if idx % 3 == 2:
                self.c.showPage()
            idx += 1


    def draw_lines(self, draw_type = 'UF', load_idx_name = None):
        '''
        Draw the defined lines.
        :return:
        '''

        points = self.data._point_dict
        lines = self.data._line_dict
        if self.data._new_buckling_method.get() == 'DNV-RP-C201 - prescriptive':
            colors = self.data.get_color_and_calc_state()['colors']
        elif self.data._new_buckling_method.get() == 'DNV PULS':
            colors = self.data.get_color_and_calc_state()['PULS colors']
        else:
            colors = self.data.get_color_and_calc_state()['ML buckling colors']

        highest_y = max([coord[1] for coord in points.values()])
        highest_x = max([coord[0] for coord in points.values()])

        if any([highest_x == 0, highest_y == 0]):
            scale = 10
        elif load_idx_name is not None:
            scale = 5
        else:
            scale = min(500/highest_y, 500/highest_x, 10)
        if draw_type == 'UF':
            origo = (50,350)
        elif load_idx_name is not None:
            origo = (50, 600 - 200*load_idx_name[0])
        else:
            origo = (50, 450)
        self.c.setLineWidth(2)
        self.c.setStrokeColor('red')
        idx, drawed_data = 0, list()
        all_line_data = self.data.get_color_and_calc_state()
        for line, pt in lines.items():
            if draw_type == 'UF':
                if self.data._new_buckling_method.get() == 'DNV-RP-C201 - prescriptive':
                    try:
                        self.c.setStrokeColor('red' if 'red' in colors[line].values() else 'green')
                    except KeyError:
                        self.c.setStrokeColor('black')
                elif self.data._new_buckling_method.get() == 'DNV PULS':
                    try:
                        method = self.data._line_to_struc[line][0].Plate.get_puls_method()
                        if self.data._PULS_results is not None:
                            util = self.data._PULS_results.get_utilization(line, method, self.data._new_puls_uf.get())
                            if util is not None:
                                self.c.setStrokeColor('red' if util > 1 else 'green')
                    except KeyError:
                        self.c.setStrokeColor('black')
                else:

                    method = self.data._line_to_struc[line][0].Plate.get_puls_method()
                    self.c.setStrokeColor(colors[line][method])

            elif draw_type == 'section' and self.data._line_to_struc[line][0].Stiffener is not None:
                self.c.setStrokeColor(all_line_data['color code']['lines'][line]['section'])
                if self.data._line_to_struc[line][0].Stiffener.get_beam_string() not in drawed_data:
                    textobject = self.c.beginText()
                    if 400 - 20 * idx > 20:
                        textobject.setTextOrigin(50, 400 - 20 * idx)
                    else:
                        textobject.setTextOrigin(300, 400 - 20 * idx)
                    textobject.setFillColor(all_line_data['color code']['lines'][line]['section'])
                    textobject.setFont("Helvetica-Oblique", 10)
                    textobject.textLine(self.data._line_to_struc[line][0].Stiffener.get_beam_string())
                    self.c.drawText(textobject)
                    drawed_data.append(self.data._line_to_struc[line][0].Stiffener.get_beam_string())
                    idx += 1
            elif draw_type == 'plate':
                self.c.setStrokeColor(all_line_data['color code']['lines'][line]['plate'])
            elif draw_type == 'pressure':
                self.c.setStrokeColor(all_line_data['color code']['lines'][line]['pressure color'])
            elif draw_type == 'utilization':
                if self.data._new_buckling_method.get() == 'DNV-RP-C201 - prescriptive':
                    self.c.setStrokeColor(all_line_data['color code']['lines'][line]['rp uf color'])
                elif self.data._new_buckling_method.get() == 'DNV PULS':
                    self.c.setStrokeColor(all_line_data['color code']['lines'][line]['PULS uf color'])
                else:
                    puls_method = self.data._line_to_struc[line][0].Plate.get_puls_method()
                    self.c.setStrokeColor(matplotlib_colors.rgb2hex(all_line_data['ML buckling colors'][line][puls_method]))

            elif draw_type == 'sigma x':
                self.c.setStrokeColor(all_line_data['color code']['lines'][line]['sigma x'])
            elif draw_type == 'sigma y1':
                self.c.setStrokeColor(all_line_data['color code']['lines'][line]['sigma y1'])
            elif draw_type == 'sigma y2':
                self.c.setStrokeColor(all_line_data['color code']['lines'][line]['sigma y2'])
            elif draw_type == 'tau xy':
                self.c.setStrokeColor(all_line_data['color code']['lines'][line]['tau xy'])
            elif draw_type == 'structure type':
                self.c.setStrokeColor(all_line_data['color code']['lines'][line]['structure type'])
            elif load_idx_name is not None:
                points = self.data._point_dict
                highest_y = max([coord[1] for coord in points.values()])
                highest_x = max([coord[0] for coord in points.values()])

                if any([highest_x == 0, highest_y == 0]):
                    scale = 10
                else:
                    scale = min(180 / highest_y, 300 / highest_x, 10)
                if line in self.data._load_dict[load_idx_name[1]][1]:
                    self.c.setStrokeColor('orange')
                else:
                    self.c.setStrokeColor('black')

            x1, y1 = points['point'+str(pt[0])][0] * scale + origo[0], \
                     points['point'+str(pt[0])][1] * scale + origo[1]
            x2, y2 = points['point'+str(pt[1])][0] * scale + origo[0], \
                     points['point'+str(pt[1])][1] * scale + origo[1]
            self.c.line(x1,y1,x2,y2)
            if load_idx_name is None:
                textobject = self.c.beginText()
                textobject.setTextOrigin(x1+(x2-x1)*0.5-5, y1 + (y2-y1)*0.5+2 )
                textobject.setFont("Helvetica-Oblique", 9)
                textobject.textLine(str(hlp.get_num(line)))
                self.c.drawText(textobject)

        if draw_type == 'UF':
            pass
        elif draw_type == 'section':
            textobject = self.c.beginText()
            textobject.setTextOrigin(50,800)
            textobject.setFont("Helvetica-Oblique", 15)
            textobject.setFillColor('black')
            textobject.textLine('Model beam section properties')
            self.c.drawText(textobject)
        elif draw_type == 'plate':
            textobject = self.c.beginText()
            textobject.setTextOrigin(50, 800)
            textobject.setFillColor('black')
            textobject.setFont("Helvetica-Oblique", 15)
            textobject.textLine('Model plate thicknesses')
            self.c.drawText(textobject)
            all_thicknesses =self.data.get_color_and_calc_state()['color code']['all thicknesses']
            for idx, thk in enumerate(all_thicknesses):
                textobject = self.c.beginText()
                if 400 - 20 * idx > 20:
                    textobject.setTextOrigin(50, 400 - 20 * idx)
                else:
                    textobject.setTextOrigin(300, 400 - 20 * idx)

                textobject.setFillColor(matplotlib.colors.rgb2hex(cmap_sections(all_thicknesses.index(thk)/
                                                                                len(all_thicknesses))))
                textobject.setFont("Helvetica-Oblique", 10)
                textobject.textLine(str(thk*1000) + ' mm')
                self.c.drawText(textobject)
        elif draw_type == 'pressure':
            textobject = self.c.beginText()
            textobject.setTextOrigin(50, 800)
            textobject.setFillColor('black')
            textobject.setFont("Helvetica-Oblique", 15)
            textobject.textLine('Highest pressures for lines in model')
            self.c.drawText(textobject)
            idx = 0

            pressure_map =self.data.get_color_and_calc_state()['color code']['pressure map']
            for press in pressure_map:
                textobject = self.c.beginText()
                if 400 - 20 * idx > 20:
                    textobject.setTextOrigin(50, 400 - 20 * idx)
                else:
                    textobject.setTextOrigin(300, 400 - 20 * idx)

                textobject.setFillColor(matplotlib.colors.rgb2hex(cmap_sections(list(pressure_map).index(press)/
                                                                                len(list(pressure_map)))))
                textobject.setFont("Helvetica-Oblique", 10)
                textobject.textLine(str(press))
                self.c.drawText(textobject)
                drawed_data.append(press)
                idx += 1
        elif draw_type == 'utilization':
            textobject = self.c.beginText()
            textobject.setTextOrigin(50, 800)
            textobject.setFillColor('black')
            textobject.setFont("Helvetica-Oblique", 12)
            if self.data._new_buckling_method.get() == 'DNV-RP-C201 - prescriptive':
                this_text = 'DNV-RP-C201 Buckling Strength of Plated Structures'
            elif self.data._new_buckling_method.get() == 'DNV PULS':
                this_text = 'Utilization factors (max of all checks) - PULS (Panel Ultimate Limit State)'
            else:
                this_text = 'ML-CL utilization factors not avaliable. ML-CLassifier only shows ok or not ok.'

            textobject.textLine(this_text)
            self.c.drawText(textobject)

            if self.data._new_buckling_method.get() == 'DNV-RP-C201 - prescriptive':
                all_utils = all_line_data['color code']['utilization map']
            elif self.data._new_buckling_method.get() == 'DNV PULS':
                all_utils = all_line_data['color code']['PULS utilization map']
            else:
                all_utils = list()

            for idx, uf in enumerate(all_utils):
                textobject = self.c.beginText()
                if 400 - 20 * idx > 20:
                    textobject.setTextOrigin(50, 400 - 20 * idx)
                else:
                    textobject.setTextOrigin(300, 400 - 20 * idx)
                textobject.setFillColor(matplotlib.colors.rgb2hex(cmap_sections(uf)))
                textobject.setFont("Helvetica-Oblique", 10)
                textobject.textLine(str('UF = ' +str(round(uf,1))))
                self.c.drawText(textobject)
        elif draw_type == 'structure type':
            textobject = self.c.beginText()
            textobject.setTextOrigin(50,800)
            textobject.setFont("Helvetica-Oblique", 15)
            textobject.setFillColor('black')
            textobject.textLine('Structure types')
            self.c.drawText(textobject)
            for idx, value in enumerate(list(all_line_data['color code']['structure types map'])):
                textobject = self.c.beginText()
                if 400 - 20 * idx > 20:
                    textobject.setTextOrigin(50, 400 - 20 * idx)
                else:
                    textobject.setTextOrigin(300, 400 - 20 * idx)

                textobject.setFillColor(matplotlib.colors.rgb2hex(cmap_sections(
                    list(all_line_data['color code']['structure types map']).index(value)/
                    len(list(all_line_data['color code']['structure types map'])))))
                textobject.setFont("Helvetica-Oblique", 10)
                textobject.textLine(str(value))
                self.c.drawText(textobject)
                drawed_data.append(value)
        elif draw_type == 'sigma x':
            textobject = self.c.beginText()
            textobject.setTextOrigin(50,800)
            textobject.setFont("Helvetica-Oblique", 15)
            textobject.setFillColor('black')
            textobject.textLine('Global stresses - sigma x')
            self.c.drawText(textobject)
            for idx, value in enumerate(list(all_line_data['color code']['sigma x map'])):
                textobject = self.c.beginText()
                if 400 - 20 * idx > 20:
                    textobject.setTextOrigin(50, 400 - 20 * idx)
                else:
                    textobject.setTextOrigin(300, 400 - 20 * idx)

                textobject.setFillColor(matplotlib.colors.rgb2hex(cmap_sections(
                    list(all_line_data['color code']['sigma x map']).index(value)/
                    len(list(all_line_data['color code']['sigma x map'])))))
                textobject.setFont("Helvetica-Oblique", 10)
                textobject.textLine(str(value))
                self.c.drawText(textobject)
                drawed_data.append(value)
        elif draw_type == 'sigma y1':
            textobject = self.c.beginText()
            textobject.setTextOrigin(50,800)
            textobject.setFont("Helvetica-Oblique", 15)
            textobject.setFillColor('black')
            textobject.textLine('Global stresses - sigma y1')
            self.c.drawText(textobject)
            for idx, value in enumerate(list(all_line_data['color code']['sigma y1 map'])):
                textobject = self.c.beginText()
                if 400 - 20 * idx > 20:
                    textobject.setTextOrigin(50, 400 - 20 * idx)
                else:
                    textobject.setTextOrigin(300, 400 - 20 * idx)

                textobject.setFillColor(matplotlib.colors.rgb2hex(cmap_sections(
                    list(all_line_data['color code']['sigma y1 map']).index(value)/
                    len(list(all_line_data['color code']['sigma y1 map'])))))
                textobject.setFont("Helvetica-Oblique", 10)
                textobject.textLine(str(value))
                self.c.drawText(textobject)
                drawed_data.append(value)
        elif draw_type == 'sigma y2':
            textobject = self.c.beginText()
            textobject.setTextOrigin(50,800)
            textobject.setFont("Helvetica-Oblique", 15)
            textobject.setFillColor('black')
            textobject.textLine('Global stresses - sigma y2')
            self.c.drawText(textobject)
            for idx, value in enumerate(list(all_line_data['color code']['sigma y2 map'])):
                textobject = self.c.beginText()
                if 400 - 20 * idx > 20:
                    textobject.setTextOrigin(50, 400 - 20 * idx)
                else:
                    textobject.setTextOrigin(300, 400 - 20 * idx)

                textobject.setFillColor(matplotlib.colors.rgb2hex(cmap_sections(
                    list(all_line_data['color code']['sigma y2 map']).index(value)/
                    len(list(all_line_data['color code']['sigma y2 map'])))))
                textobject.setFont("Helvetica-Oblique", 10)
                textobject.textLine(str(value))
                self.c.drawText(textobject)
                drawed_data.append(value)
        elif draw_type == 'tau xy':
            textobject = self.c.beginText()
            textobject.setTextOrigin(50,800)
            textobject.setFont("Helvetica-Oblique", 15)
            textobject.setFillColor('black')
            textobject.textLine('Global stresses - tau xy')
            self.c.drawText(textobject)
            for idx, value in enumerate(list(all_line_data['color code']['tau xy map'])):
                textobject = self.c.beginText()
                if 400 - 20 * idx > 20:
                    textobject.setTextOrigin(50, 400 - 20 * idx)
                else:
                    textobject.setTextOrigin(300, 400 - 20 * idx)

                textobject.setFillColor(matplotlib.colors.rgb2hex(cmap_sections(
                    list(all_line_data['color code']['tau xy map']).index(value)/
                    len(list(all_line_data['color code']['tau xy map'])))))
                textobject.setFont("Helvetica-Oblique", 10)
                textobject.textLine(str(value))
                self.c.drawText(textobject)
                drawed_data.append(value)
        elif load_idx_name is not None:
            for lidx, loadtext in enumerate(reversed(self.data._load_dict[load_idx_name[1]][0].get_report_string())):
                textobject = self.c.beginText()
                textobject.setTextOrigin(370 , origo[1]+50+ 11*lidx)
                textobject.setFont("Helvetica-Oblique", 11)
                textobject.setFillColor('black')
                textobject.textLine(loadtext)
                self.c.drawText(textobject)

    def coord(self, x, y, unit=1):
        """
        # http://stackoverflow.com/questions/4726011/wrap-text-in-a-table-reportlab
        Helper class to help position flowables in Canvas objects
        """
        x, y = x * unit, self.height - y * unit
        return x, y

        # ----------------------------------------------------------------------

    def createParagraph(self, ptext, x, y, style=None):
        """"""
        if not style:
            style = self.styles["Normal"]
        p = Paragraph(ptext, style=style)
        p.wrapOn(self.c, self.width, self.height)
        p.drawOn(self.c, *self.coord(x, y, mm))

    # ----------------------------------------------------------------------
    def savePDF(self):
        """"""
        self.c.save()

    # ----------------------------------------------------------------------

    def createTable(self):
        '''
        Create a table of results for all lines.
        '''

        table_all = []
        headers = ['Line', 'pl thk', 's', 'web h', 'web thk', 'fl. w', 'fl. thk', 'sig x1', 'sig x2', 'sig y1',
                   'sig y2', 'tau xy', 'max press.', 'sec. mod', 'min sec.', 'min plt',
                   'shr area', 'min shr A', 'fat uf', 'buc uf']
        table_all.append(headers)
        for line in sorted(self.data._line_dict.keys()):
            struc_obj = self.data._line_to_struc[line][0]
            pressure = round(self.data.get_highest_pressure(line)['normal'] / 1000,0)

            if self.data._PULS_results is not None:
                puls_method = self.data._line_to_struc[line][0].Plate.get_puls_method()
                if line in self.data._PULS_results.get_run_results().keys():
                    if puls_method == 'buckling':
                        buckling_uf = \
                        self.data._PULS_results.get_run_results()[line]['Buckling strength']['Actual usage Factor'][0]
                    else:
                        buckling_uf = \
                        self.data._PULS_results.get_run_results()[line]['Ultimate capacity']['Actual usage Factor'][0]
            else:
                buckling_uf = str(round(max(self.data.get_color_and_calc_state()['buckling'][line]), 2))

            if self.data.get_color_and_calc_state()['fatigue'][line]['damage'] is not None:
                fat_uf = self.data.get_color_and_calc_state()['fatigue'][line]['damage']
                fat_uf = round(fat_uf, 3)
            else:
                fat_uf = self.data.get_color_and_calc_state()['fatigue'][line]['damage']


            data = [line,str(struc_obj.Plate.get_pl_thk() * 1000), str(struc_obj.Plate.get_s() * 1000),
                    str('' if struc_obj.Stiffener is None else struc_obj.Stiffener.get_web_h() * 1000),
                    str('' if struc_obj.Stiffener is None else struc_obj.Stiffener.get_web_thk() * 1000),
                    str('' if struc_obj.Stiffener is None else struc_obj.Stiffener.get_fl_w() * 1000),
                    str('' if struc_obj.Stiffener is None else struc_obj.Stiffener.get_fl_thk() * 1000),
                    str(round(struc_obj.Plate.get_sigma_x1(), 0)), str(round(struc_obj.Plate.get_sigma_x2(), 0)),
                    str(round(struc_obj.Plate.get_sigma_y1(), 0)),
                    str(round(struc_obj.Plate.get_sigma_y2(), 0)),
                    str(round(struc_obj.Plate.get_tau_xy(), 0)), str(round(pressure, 2) * 1000),
                    str(int(min(self.data.get_color_and_calc_state()['section_modulus'][line]['sec_mod']) * 1000 ** 3)),
                    str(int(self.data.get_color_and_calc_state()['section_modulus'][line]['min_sec_mod'] * 1000 ** 3)),
                    str(round(self.data.get_color_and_calc_state()['thickness'][line]['min_thk'], 2)),
                    str(int(self.data.get_color_and_calc_state()['shear_area'][line]['shear_area'] * 1000 ** 2)),
                    str(int(self.data.get_color_and_calc_state()['shear_area'][line]['min_shear_area'] * 1000 ** 2)),
                    fat_uf, buckling_uf]

            table_all.append(data)

        t = Table(table_all,colWidths=[0.55*inch])
        t.setStyle(TableStyle([
            ('GRID', (0, 0), (-1, -1), 0.5, colors.gray),
            ('BACKGROUND', (0, 0), (-1, -1), colors.lightblue),
            ('FONTSIZE', (0, 0), (-1, -1), 8),
            ('FONTSIZE', (0, 4), (-1, 4), 8),
            ('TEXTFONT', (0, 1), (-1, 1), 'Times-Bold'),
            ('TEXTFONT', (0, 4), (-1, 4), 'Times-Bold'),
        ]))

        return [t,]


if __name__ == '__main__':
    import multiprocessing, ctypes, tkinter
    import main_application as app
    multiprocessing.freeze_support()
    errorCode = ctypes.windll.shcore.SetProcessDpiAwareness(2)
    root = tkinter.Tk()
    my_app = app.Application(root)
    ship_example = r'C:\Github\ANYstructure\ship_section_example.txt'
    my_app.openfile(ship_example)
    #my_app.table_generate()
    my_app.report_generate(autosave=True)
    # doc = LetterMaker("example.pdf", "The MVP", 10, to_report_gen)
    # doc.createDocument()
    # doc.savePDF()




"""A setuptools based setup module.
See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

# Arguments marked as "Required" below must be included for upload to PyPI.
# Fields marked as "Optional" may be commented out.

def readme():
    with open('README.rst') as file:
        return file.read()

setup(
    name='ANYstructure',  # Required
    url = 'https://github.com/audunarn/ANYstructure',
    entry_points={"console_scripts": ['ANYstructure = any_files.__main__:main']},
    version='4.6.14',  # Required
    license='MIT',
    description='A plate field optimization tool for offshore structures calculated according to DNV standards',
    long_description = readme(),
    author='Audun Arnesen Nyhus',  # Optional
    author_email='audunarn@gmail.com',  # Optional
    classifiers=[  # Optional
        'Development Status :: 5 - Production/Stable',
        'Environment :: X11 Applications',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.11',
        'Topic :: Scientific/Engineering'],
    keywords='dnvgl-gl-os-c101 naval_architecture structural_engineering steel buckling fatigue local_scantlings optimization weight',
    include_package_data=True,
    install_requires=['scipy', 'numpy', 'matplotlib', 'reportlab', 'xlwings', 'sklearn', 'scikit-learn'],
    packages=['any_files'],
    py_modules = [],
)